unit NPCX;

interface

uses NGlob,
     Crt,
     NGraph;

procedure DisplayPCX (FileName: String);
procedure SetPCXPal (FileName: String);
procedure NormPalette;

type
  {   PCX 䠩 }
  TPCXHeader = record
    Manuf          : Byte;                  { ᥣ = 10 }
    Hard           : Byte;                  { Version information         }
    Cod            : Byte;                  { Run-length encoding (=1)    }
    BitPrePix      : Byte;                  { Bits per pixel              }
    X1             : Word;                  { Picture dimensions (incl)   }
    Y1             : Word;                  {                             }
    X2             : Word;                  {                             }
    Y2             : Word;                  {                             }
    HorizRes       : Word;                  { Display horiz resolution    }
    VertRes        : Word;                  { Display vert  resolution    }
    Pal            : array[1..48] of Byte;  { Pallete                     }
    VMode          : Byte;                  { (ignored)                   }
    NUmbOFPlan     : Byte;                  { Number of planes (ver 2.5=0)}
    BytePerLine    : Word;                  { Bytes per line              }
    Col            : Word;                  { Palette Info (1=col, 2=gray)}
    ScanHor        : Word;                  { Scanner resolution          }
    ScanVer        : Word;                  {                             }
    Extra          : array[1..54] of Byte;  { Extra space (filler)        }
  end;

implementation

procedure DisplayPCX (FileName: String);
var
  PCXFile: File;
  Header: TPCXHeader;
  Body: Pointer;
  NumRead, ByteCnt,
  DispStr, DispByte, CurShift: Word;
  Plan, Numb, RealCount: Byte;
  Value: Byte;
  DataP, CarPtr: ^Byte;
  Change: Boolean;
  BuffSize, i: Word;

procedure PCXEnd;
begin
  Close (PCXFile);
  FreeMem (Body, BuffSize);
end;

procedure PCXBegin;
begin
  Assign (PCXFile, FileName);
  Reset (PCXFile, 1);
  BlockRead (PCXFile, Header, SizeOf (TPCXHeader), NumRead);
  BuffSize := 10000;
  GetMem (Body, BuffSize);
  CarPtr := ActBlPtr;
  DispByte := 0;
  Plan := 0;
  DispStr := 0;
end;

procedure UnPack;
begin
  if (DataP^ >= $C0) then begin
    Numb := DataP^ - $C0;
    Inc (LongInt (DataP)); Inc (ByteCnt);
    if (ByteCnt >= NumRead) then begin
      Seek (PCXFile, FilePos (PCXFile) - 1);
      Numb := 0;
    end;
    Value := DataP^;
    Inc (LongInt (DataP)); Inc (ByteCnt);
  end
  else begin
    Numb := 1;
    Value := DataP^;
    Inc (LongInt (DataP)); Inc (ByteCnt);
  end;
end;

procedure ChangePlan;
begin
  DispByte := 0;
  Inc (Plan);
  if (Plan = Header.NumbOfPlan) then begin
    Plan := 0;
    Inc (DispStr);
    Inc (LongInt (CarPtr), Header.HorizRes);
  end;
end;

procedure EOLControl;
begin
  if (DispByte + Numb < Header.BytePerLine) then begin
    RealCount := Numb;
    Numb := 0;
    Change := False;
  end
  else begin
    RealCount := Header.BytePerLine - DispByte;
    Numb := Numb - RealCount;
    Change := True;
  end;
end;

begin
  PCXBegin;
  while (NumRead <> 0) do begin
    BlockRead (PCXFile, Body^, BuffSize, NumRead);
    DataP := Addr (Body^);
    ByteCnt := 0;
    while (ByteCnt < NumRead) do begin
      UnPack;
      while (Numb > 0) do begin
        EOLControl;
          for i:=1 to RealCount do begin
            CurShift := DispByte;
            asm
              les   di, CarPtr
              add   di, CurShift
              mov   al, Value
              mov   byte ptr es:[di], al
            end;
            Inc (DispByte);
          end;
        if Change then begin
          ChangePlan;
          if (DispStr = Header.VertRes) then begin
            PCXEnd;
            Exit;
          end;
        end;
      end;
    end;
  end;
  PCXEnd;
end;

procedure NormPalette;
var
  Pal: array [1..768] of Byte absolute Palette;
  i: Word;
begin
  for i:=1 to 768 do Pal[i] := Pal[i] shr 2;
end;

procedure SetPCXPal (FileName: String);
var
  PCXFile: File;
  NumRead: Word;
  Ident, i: Byte;
begin
  Assign (PCXFile, FileName);
  Reset (PCXFile, 1);
  Seek (PCXFile, FileSize (PCXFile) - 769);
  BlockRead (PCXFile, Ident, 1, NumRead);
  BlockRead (PCXFile, Palette, 768, NumRead);
  Close (PCXFile);
  NormPalette;
end;

end.


