unit NSprite;

interface

uses NGlob,
     Dos,
     NGraph;

const
  SprPhMax = 16;

type
  PSprDta = ^TSprDta;
  TSprDta = record
    NofPhase: Byte;
    SprPtr: array [1..SprPhMax] of Pointer;
    Sz: array [1..SprPhMax] of Word;
  end;

  PSprXY = ^TSprXY;
  TSprXY = record
    X, Y: Integer;
  end;

procedure LoadSprite (var S: TSprDta; FlNm: PathStr);
procedure FreeSprite (var S: TSprDta);
procedure PutSprite (PSpr: Pointer; X, Y: Integer);
procedure PutVPSprite (PSpr: Pointer; X1, Y1: Integer);

implementation

procedure LoadSprite (var S: TSprDta; FlNm: PathStr);
var
  NB, NumRead: Word;
  i: Byte;
  Handle: File;
begin
  Assign (Handle, FlNm);
  ReSet (Handle, 1);
  if IOResult <> 0 then ErrMsg ('File not found '+ FlNm);
  i := 0;
  with S do
    repeat
      BlockRead (Handle, NB, 2, NumRead);
      if NumRead <> 2 then begin
        NOfPhase := i;
        Close (Handle);
        Exit;
      end;
      Inc (i);
      Sz[i] := NB;
      TGetMem (SprPtr[i], NB);
      BlockRead (Handle, SprPtr[i]^, NB, NumRead);
      if NumRead <> NB then ErrMsg ('Reading error in '+ FlNm);
    until FALSE;
end;

procedure FreeSprite (var S: TSprDta);
var
  i: Byte;
begin
  with S do
    for i:=1 to NofPhase do
      FreeMem (SprPtr[i], Sz[i]);
end;

procedure PutSprite (PSpr: Pointer; X, Y: Integer);
var
  XCount, YCount, XInc: Integer;
begin
  XCount := PSprXY(PSpr)^.X;
  YCount := PSprXY(PSpr)^.Y;
  XInc := ActBX - XCount;
  asm
    les  di, ActBlPtr
    mov  ax, Y
    mov  bx, ActBX
    mul  bx
    add  ax, X
    add  di, ax
    mov  dx, YCount
    push ds
    lds  si, PSpr
    add  si, 4
    mov  ah, 0
@@NxtStr:
    mov  cx, XCount
@@NxtByte:
    lodsb
    cmp  al, ah
    je   @@Hole
    stosb
    jmp  @@Fill
@@Hole:
    inc  di
@@Fill:
    loop @@NxtByte
    dec  dx
    add  di, XInc
    cmp  dx, 0
    jg   @@NxtStr
    pop  ds
  end;
end;

procedure PutVPSprite (PSpr: Pointer; X1, Y1: Integer);
var
  X2, Y2: Integer;
  XCount, YCount, IncX,
  PreX, PreY, PostX, PostY: Word;
begin
  XCount := PSprXY (PSpr)^.X;
  YCount := PSprXY (PSpr)^.Y;
  PreX := 0; PreY := 0;
  PostX := 0; PostY := 0;
  X2 := X1 + XCount; Y2 := Y1 + YCount;
  if X1 >= ActBX then Exit;
  if Y1 >= ActBY then Exit;
  if X1 <= 0     then begin PreX := -X1; XCount := XCount - PreX; X1 := 0 end;
  if Y1 <= 0     then begin PreY := -Y1; YCount := YCount - PreY; Y1 := 0 end;
  if X2 <= 0     then Exit;
  if Y2 <= 0     then Exit;
  if X2 >= ActBX then begin PostX := X2 - ActBX; XCount := XCount - PostX; end;
  if Y2 >= ActBY then begin PostY := Y2 - ActBY; YCount := YCount - PostY; end;
  PreY := PreY * PSprXY (PSpr)^.X;
  IncX := ActBX - XCount;
  asm
    les  di, ActBlPtr
    mov  ax, Y1
    mov  bx, ActBX
    mul  bx
    add  di, ax
    add  di, X1
    mov  dx, YCount
    push ds
    lds  si, PSpr
    add  si, 4
    add  si, PreY
@@NxtStr:
    mov  cx, XCount
    add  si, PreX
@@NxtByte:
    mov  al, byte ptr ds:[si]
    cmp  al, 0
    je   @@Hole
    mov  byte ptr es:[di], al
@@Hole:
    inc  si
    inc  di
    loop @@NxtByte
    add  si, PostX
    dec  dx
    add  di, IncX
    cmp  dx, 0
    jg   @@NxtStr
@@Done:
    pop  ds
  end;
end;

end.






